import ij.*;
import ij.text.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;


import java.util.*;
import java.awt.image.*;
import java.awt.color.*;

/** 22/05/02 E. Glory 	last modification 16/11/2003    
    The Lab color space is approximately uniform designed by the CIE (1931)
    1st component L = black/White             (luminance)
    2nd component a = red/green               (a chromatic component )
    3rd component b = blue/yellow             (b chromatic component)
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p83-84
*/

public class fromRGBtoLab_ implements PlugInFilter {
		ImagePlus imp;
        public  int [] Pixels;
        int illuminant;
  //int compt=0;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();			
		this.Pixels=new int[h*w];	
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		Pixels=(int[])ip.getPixels();

/*		String illuminantsrc="D65";
		switch (illuminantsrc){
		case "D65" : illuminant=0; break;
		case "F2" : illuminant=1; break;    // cool white
		case "F11" : illuminant=2; break;   // TL 84
		case "A" : illuminant=3; break;   
		case "C" : illuminant=4; break;   
		case "D50" : illuminant=5; break;   
		default : break;
		}
  */
		stack(ip);
	}

   	public void stack(ImageProcessor ip) {
	    IJ.showProgress(0.1);

	    String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));

        byte[] L=new byte[this.Pixels.length];
		byte[] a=new byte[this.Pixels.length];
		byte[] b=new byte[this.Pixels.length];

		this.getLab(L,a,b);
		//affiche(L,a,b,Pixels);
		
		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);

		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_L",L);
		stack.addSlice(name+"_a",a);
		stack.addSlice(name+"_b",b);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    	imp2.show();
	}

    public void getLab(byte[] L, byte[] a, byte[] b) {
	    int c,R,G,B;
		float[] Lab = new float[3];
		
		for (int i=0; i < Pixels.length; i++) {
			c = Pixels[i];
		  	
		  	R = (c&0xff0000)>>16;
		  	G = (c&0xff00)>>8;
		  	B = c&0xff;
		  	
		  	Lab=RGBtoLab(R,G,B,Lab);
		  	
		  	//if(i<=2)System.out.println(R+"\t"+G+"\t"+B+"\t"+Lab[0]);
		  	/*
		  	TextWindow tw = new TextWindow("Control","img "+"level","",300,400);
		  	tw.append("color : "+R+","+G+","+B+")");
		  	tw.append("avant norma L"+Lab[0]);
		  	tw.append("avant norma a"+Lab[1]);
		  	tw.append("avant norma b"+Lab[2]);
		  	tw.append("");
		  	*/
		  
		    /*normalisation*/   
		  	L[i] = (byte)((int)(Lab[0]*0.354366597));     // [0;255]   0,354366597 = 255/7219.593782;
		  	a[i] = (byte)((int)((Lab[1]+3170.662855)*255/(2*3170.66285))); // [0;255] 
		  	b[i] = (byte)((int)((Lab[2]+1268.26514)*255/(2*1268.26514)));  // [0;255] 
		  	/*
		  	L[i] = (byte)((int)(Lab[0]*255f/8.9914424f));   // [0;255]
		  	a[i] = (byte)((int)(Lab[1]*1.1838f+127.5f));    // [0;255]
		  	b[i] = (byte)((int)(Lab[2]*2.956f+127.5f));     // [0;255]
		  	*/
		  	/*
			tw.append("apres norma "+XYZvals[0]);
			int l_ = L[i]&0xff;
			int a_ = a[i]&0xff;
			int b_ = b[i]&0xff;
			tw.append("apres norma L"+l_);
			tw.append("apres norma a"+a_);
		  	tw.append("apres norma b"+b_);
		  	*/
		}
	}


    public float[] RGBtoLab(int R, int G, int B, float[] Labvals){
	    if (Labvals==null)
    		Labvals =new float[3];

	       // ->The Colour Image Processsing Handbook, (Sangwine,Horne) p71)
	       // for the EBU RGB space, the transformation in X,Y,Z is  


		   /* illuminant D65, lumiere du jour -> sRGB*/
	       float  X = 0.412453f*R + 0.357580f*G + 0.180423f*B;  // [0 ; 242.36628]
	       float  Y = 0.212671f*R + 0.715160f*G + 0.072169f*B;  // [0 ; 255]
	       float  Z = 0.019334f*R + 0.119193f*G + 0.950227f*B;  // [0 ; 277.63227]
//	       X = X * 1.052127; // normalise [0;255]
//	       Y = Y;
//	       Z = Z * 0.918481; // normalise [0;255]
			
			/*   
	       float  X = 0.490f*R + 0.310f*G + 0.200f*B;
	       float  Y = 0.177f*R + 0.812f*G + 0.011f*B;    // [0;255]
	       float  Z =            0.010f*G + 0.900f*B;		
	       X/=255;  // [0;1]
	       Y/=255;  // [0;1]
	       Z/=255;  // [0;1]
 			*/
 			
	       //test
	       illuminant=0;
	       float X0=0,Y0=0,Z0=0;
	       // the D65 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==0){
	 		X0 = 0.950456f;
	 		Y0 = 1.0f;
		 	Z0 = 1.088754f;
	       }

	       // the F2 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==1){
			 X0 = 0.9919f;
			 Y0 = 1.0f;
			 Z0 = 0.6739f;
	       }

	       // the F11 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==2){
			 X0 = 1.0096f;
			 Y0 = 1.0f;
			 Z0 = 0.6435f;
	       }

	       // the A illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==3){
			 X0 = 1.0985f;
			 Y0 = 1.0f;
			 Z0 = 0.3556f;
	       }

	       // the C illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==4){
			 X0 = 0.9807f;
			 Y0 = 1.0f;
			 Z0 = 1.1823f;
	       }

	       // default =  the D50 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==5){
			 X0 = 0.9642f;
			 Y0 = 1.0f;
			 Z0 = 0.8249f;
		   }
	       float  fX = X/X0;
	       float  fY = Y/Y0;
	       float  fZ = Z/Z0;
 
	       Labvals[0] = computeL(fY);     // L [0; 719,593782]
	       Labvals[1] = (float)(500.*(Math.pow(fX, 1./3.) - Math.pow(fY, 1./3.))) ;      // a [-3170,66285;3170,66285]
	       Labvals[2] = (float)(200.*(Math.pow(fY, 1./3.) - Math.pow(fZ, 1./3.))) ;      // b [-1268,26514;1268,26514]
  
  		 
  		   //if(compt<=2)System.out.println("\tY="+Y+"\t"+Y0+"\tfY="+fY+"\t");
		   //compt++;
	       return Labvals;
	 }

     public float computeL (float fY){
	   	   float result;
		   float epsilon = 216.f/24389.f;	
			
		   if(fY>epsilon)
		     result=(float)(116.*Math.pow(fY, 1./3.)-16);
		   else
		     result= (epsilon*fY);

		   return result;
     } 
         
                
     public void affiche(byte[] L, byte[] a, byte[] b, int[] Pixels) {
	    TextWindow tw = new TextWindow("Control","img "+"level","",300,400); 
		for (int i=0; i<Pixels.length; i++){
			int l_=L[i]&0xff;
			int a_=a[i]&0xff;
			int b_=b[i]&0xff;
			tw.append("pixel "+i+" : Lab("+l_+","+a_+","+b_+")");
		}
	 }
}
