import ij.*;
import ij.text.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;

/** 22/05/02 E. Glory 	last modification 17/11/2003 
   
    The Luv color space is uniform and recommended for applications in additive light source condition (CIE) 
    1st component L =              (luminance)
    2nd component u =              (a chromatic component )
    3rd component v =              (a chromatic component)
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p85-86
*/

public class fromRGBtoLuv_ implements PlugInFilter {
		ImagePlus imp;
        public  int [] Pixels;
        int illuminant;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();			
		this.Pixels=new int[h*w];	
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		Pixels=(int[])ip.getPixels();
		stackOpp(ip);
		//computeLimitsUV();
	}

   	public void stackOpp(ImageProcessor ip) {
	    IJ.showProgress(0.1);

	    String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));

        byte[] L=new byte[this.Pixels.length];
		byte[] u=new byte[this.Pixels.length];
		byte[] v=new byte[this.Pixels.length];

		this.getLuv(L,u,v);
		//affiche(L,u,v,Pixels);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);

		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_L",L);
		stack.addSlice(name+"_u",u);
		stack.addSlice(name+"_v",v);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    	imp2.show();
	}

    public void getLuv(byte[] L, byte[] u, byte[] v) {
	    int c,R,G,B;
		float[] Luv = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  
		  Luv=RGBtoLuv(R,G,B,Luv);
		  
		  /*normalisation*/
		  /*TextWindow tw = new TextWindow("Control","img "+"level","",300,400);
		  tw.append("color : "+R+","+G+","+B+")");
		  tw.append("avant norma L"+Luv[0]);
		  tw.append("avant norma u"+Luv[1]);
		  tw.append("avant norma v"+Luv[2]);
		  tw.append("");
		  	*/
		  L[i] = (byte)((int)(Luv[0]*0.354366597));    // [0;255] // 0.354366597= 255f/8.9914424f));   
		  u[i] = (byte)((int)((Luv[1]+1850.73205)*0.028952107)); // [0;255] // =(a+1850,73205)*255/(1850,73205+6956,917)
		  v[i] = (byte)((int)((Luv[2]+1316.51339)*0.100073175)); // [0;255] //=(b+1316,51339)*255/(1316,51339+1231.622)
		  
		  /*
		  int l_ = L[i]&0xff;
		  int u_ = u[i]&0xff;
		  int v_ = v[i]&0xff;
		  
		  tw.append("apres norma L"+l_);
		  tw.append("apres norma u"+u_);
		  tw.append("apres norma v"+v_);
		  */
		}
	 }


         public float[] RGBtoLuv(int R, int G, int B, float[] Luvvals){
	       if (Luvvals==null)
    		  Luvvals =new float[3];
	       

	       // ->The Colour Image Processsing Handbook, (Sangwine,Horne) p71)
	       // for the EBU RGB space, the transformation in X,Y,Z is  

	       float  X = 0.412453f*R + 0.357580f*G + 0.180423f*B; // [0 ; 242.36628]
	       float  Y = 0.212671f*R + 0.715160f*G + 0.072169f*B; // [0 ; 255]
	       float  Z = 0.019334f*R + 0.119193f*G + 0.950227f*B; // [0 ; 277.63227]
//	       X /= (255 * 0.950456);
//	       Y /=  255;
//	       Z /= (255 * 1.088754);  
			/*			   
	       float  X = 0.490f*R + 0.310f*G + 0.200f*B;
	       float  Y = 0.177f*R + 0.812f*G + 0.011f*B;
	       float  Z =            0.010f*G + 0.900f*B;	
		
	       X/=255;  // [0;1]
	       Y/=255;  // [0;1]
	       Z/=255;  // [0;1]
			*/
	       //test
	       illuminant=0;

	       float X0=0,Y0=0,Z0=0;
	       // the D65 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==0){
			 X0 = 0.950456f;
			 Y0 = 1.0f;
			 Z0 = 1.088754f;
	       }

	       // the F2 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==1){
			 X0 = 0.9919f;
			 Y0 = 1.0f;
			 Z0 = 0.6739f;
	       }

	       // the F11 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==2){
			 X0 = 1.0096f;
			 Y0 = 1.0f;
			 Z0 = 0.6435f;
	       }

	       // the A illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==3){
			 X0 = 1.0985f;
			 Y0 = 1.0f;
			 Z0 = 0.3556f;
	       }

	       // the C illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==4){
			 X0 = 0.9807f;
			 Y0 = 1.0f;
			 Z0 = 1.1823f;
	       }
	    // default =  the D50 illuminant has X0, Y0, Z0 coordinates :
	       if(illuminant==5){
			 X0 = 0.9642f;
			 Y0 = 1.0f;
			 Z0 = 0.8249f;
	       }

	       float  L = computeL(Y/Y0); // L [0; 719,593782]

	       float  u  = 4f*X /(X +15f*Y +3f*Z );
	       float  u0 = 4f*X0/(X0+15f*Y0+3f*Z0);
	       
	       float  v  = 9f*Y /(X +15f*Y +3f*Z );
 	       float  v0 = 9f*Y0/(X0+15f*Y0+3f*Z0);

	       Luvvals[0] = L;                  // [0; 719,593782]
	       Luvvals[1] = 13f*L*(u-u0);       // [-1850,73205;6956,917]
	       Luvvals[2] = 13f*L*(v-v0);   	// [-1316,51339;1231.62174]

	       return Luvvals;
	 }

     public float computeL (float fY){
	   	   float result;
		   float epsilon = 216.f/24389.f;	
			
		   if(fY>epsilon)
		     result=(float)(116.*Math.pow(fY, 1./3.)-16);
		   else
		     result= (float)(24389.*fY/27.);

		   return result;
     } 
	
     public void affiche(byte[] L, byte[] u, byte[] v, int[] Pixels) {
	    TextWindow tw = new TextWindow("Control","img "+"level","",300,400); 
		for (int i=0; i<Pixels.length; i++){
			int l_=L[i]&0xff;
			int u_=u[i]&0xff;
			int v_=v[i]&0xff;
			tw.append("pixel "+i+" : Lab("+l_+","+u_+","+v_+")");
		}
	 }  
	 
     public void computeLimitsUV() {
	    //TextWindow tw = new TextWindow("Control","img "+"level","",300,400); 
	    
	    float X = 242.36628f;
	    float Y = 0.f;
	    float Z = 277.63227f;
	    
	    float l_= 0.f;
		float u_= 0.f;
		float v_= 0.f;
	    
		for (X=238.f; X<=242.36628f; X=X+0.2f){
			for (Y=23.5f; Y<=23.8f; Y=Y+0.01f){
				for(Z=275.f; Z<=277.63f; Z=Z+0.2f){
					
						
						l_= computeL(Y);
						//u_= computeU(X,Y,Z,l_);
						v_= computeV(X,Y,Z,l_);
						//tw.append("XY : "+X+"\t"+Y+" Lu : "+l_+"\t"+u_+"\t"+v_);
						//tw.append("XYZ: "+X+"\t "+Y+"\t "+Z+" Lv: "+l_+"\t "+v_);
					}
			}			
		//tw.append("");	
		}
	
	 }  	 
	
	 public float computeU(float X, float Y, float Z, float L) {
	    	
		float  u_  = 4f*X /(X +15f*Y +3f*Z );
   		float  u0 = 0.1978394f ; //4f*X0/(X0+15f*Y0+3f*Z0);

	    return (13f*L*(u_-u0));
	 }  	  
	 
	 public float computeV(float X, float Y, float Z, float L) {
	    	
		float  v_  = 9f*Y /(X +15f*Y +3f*Z );
   		float  v0 = 0.468342201f ; //4f*X0/(X0+15f*Y0+3f*Z0);

	    return (13f*L*(v_-v0));
	 }  
}

