import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory
    YIQ is the American television color space used (Benson, 1992).
    1st component Y = 0.299 R + 0.587 G + 0.114 B         
    2nd component I = 0.596 R - 0.274 G - 0.322 B 
    3rd component Q = 0.211 R - 0.523 G + 0.312 B
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p72
*/


public class fromRGBtoYIQ_ implements PlugInFilter {
	ImagePlus imp;
        public  int [] Pixels;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();		
		this.Pixels=new int[h*w];		
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	        IJ.showProgress(0.1);

	        String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));
	
		Pixels=(int[])ip.getPixels();
		byte[] Y=new byte[this.Pixels.length];
		byte[] I=new byte[this.Pixels.length];
		byte[] Q=new byte[this.Pixels.length];

		this.getYIQ(Y,I,Q);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);
		//imp2.trimProcessor();
		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_Y",Y);
		stack.addSlice(name+"_I",I);
		stack.addSlice(name+"_Q",Q);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    		imp2.show();

	}
       
         public void getYIQ(byte[] Y, byte[] I, byte[] Q) {
	        int c,R,G,B;
		float[] YIQ = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  YIQ=RGBtoYIQ(R,G,B,YIQ);
		  Y[i] = (byte)((int)(YIQ[0]));
		  I[i] = (byte)((int)(YIQ[1]*0.839f+127.5f)); 
		  Q[i] = (byte)((int)(YIQ[2]*0.956f+127.5f));
		}
	 }

         public float[] RGBtoYIQ(int R, int G, int B, float[] YIQvals){
	       if (YIQvals==null)
    		  YIQvals =new float[3];

	       YIQvals[0]= 0.299f*R+0.587f*G+0.114f*B;       // [0;255]
	       YIQvals[1]= 0.596f*R-0.274f*G-0.322f*B;       // [-51.98;151.98]
 	       YIQvals[2]= 0.211f*R-0.523f*G+0.312f*B;       // [-133.365;133.365]
	       return YIQvals;
	 }
}
